@@ -5,15 +5,12 @@ from .common import *
 from .logutil import *
 from .shared import Shared
 
-import importlib.resources
-from pathlib import Path
-
 
 import hail as hl
 from munch import Munch
 from datetime import datetime
 
-from cap import shared
+from copy import deepcopy
 
 if __name__ == '__main__':
     print('This module is not executable. Please import this module in your program.')
@@ -37,11 +34,10 @@ class Executor:
      
             hl.init(log=Shared.runtime.hailLog)
 
-            with importlib.resources.path('cap', 'VERSION') as path:
-                Shared.runtime.capVersion = Path(path).read_text()
             Shared.runtime.hailVersion = hl.version()
             sc = hl.spark_context()
             Shared.runtime.sparkVersion = sc.version
+            workload.Update()
      
             Log(f'CAP Version: {Shared.runtime.capVersion}')
             Log(f'Hail Version: {Shared.runtime.hailVersion}')
@@ -49,13 +45,7 @@ class Executor:
             Log(f'CAP Log {Shared.runtime.capLog}')
             Log(f'Hail Log {Shared.runtime.hailLog}')
 
-            if 'runtimes' not in workload.globConfig:
-                workload.globConfig.runtimes = list()
-            runtime = Shared.runtime
-            runtime.dateTime = str(datetime.now().strftime("%Y/%m/%d-%H:%M:%S"))
-            workload.globConfig.runtimes.append(runtime)
-
-            workload.Update()
+            
 
             LogPrint("+++++++++++++++++++++++++++++++")
             LogPrint("+++++++++++++++++++++++++++++++")
@@ -77,7 +67,6 @@ class Executor:
                 stage = workload.stages[stageId]
                 if stage.spec.status != 'Completed' or reset:
                     self.ExecuteStage(stage)
-                    workload.Update()
 
     @D_General
     def ExecuteStage(self, stage):
@@ -86,13 +75,23 @@ class Executor:
         workload.CheckStage(stage)  # Check the stage right before execution to make sure no dynamic error occurs
         LogPrint(f'Started')
         func = getattr(Operation, stage.spec.function)
-        stage.spec.runtime = Shared.runtime
-        stage.spec.startTime = datetime.now()
+        runtime = Munch()
+        runtime.base = Shared.runtime.base
+        if 'runtimes' not in stage.spec:
+            stage.spec.runtimes = list()
+        stage.spec.runtimes.append(runtime)
+        runtime.startTime = datetime.now()
+        workload.Update()
         workload.ProcessLiveInputs(stage)
+        workload.Update()
         func(stage)
+        workload.Update()
         workload.ProcessLiveOutputs(stage)
-        stage.spec.endTime = datetime.now()
-        stage.spec.execTime = str(stage.spec.endTime - stage.spec.startTime)
+        workload.Update()
+        runtime.endTime = datetime.now()
+        runtime.execTime = str(runtime.endTime - runtime.startTime)
+        runtime.status = 'Completed'
         stage.spec.status = 'Completed'
-        LogPrint(f'Completed in {stage.spec.execTime}')
+        workload.Update()
+        LogPrint(f'Completed in {runtime.execTime}')
         Shared.CurrentStageForLogging = None
